/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.demo.thermorest.cloud.mqtt;

import java.io.IOException;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

import com.microej.demo.thermorest.Messages;
import com.microej.demo.thermorest.cloud.PeriodicPublisherCloud;

import ej.util.message.Level;

/**
 * CloudConnector publishing to eclipse cloud.
 */
public class ThermostatMQTT extends PeriodicPublisherCloud implements MqttCallback {

	private static final String NAME = "MQTT"; //$NON-NLS-1$

	private MqttClient client;

	private String subscribedTopic;

	private final String clientID;

	private final String uri;

	/**
	 * Instantiates a Thermostat Cloud connector over tcp MQTT.
	 *
	 * @param clientID
	 *            The client ID.
	 * @param uri
	 *            The broker URI.
	 */
	public ThermostatMQTT(String clientID, String uri) {
		this.clientID = clientID;
		this.uri = uri;
	}

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	protected void subscribe(String topic) throws IOException {
		this.subscribedTopic = topic;
	}

	@Override
	public void messageArrived(String topic, MqttMessage message) throws Exception {
		if (topic.equals(this.subscribedTopic)) {
			receive(message.toString());
		}
	}

	@Override
	public void connectionLost(Throwable cause) {
		try {
			disconnect();
		} catch (IOException e) {
			Messages.LOGGER.log(Level.INFO, Messages.CATEGORY, Messages.CLOUD_ERROR, e);
		}
	}

	@Override
	public void deliveryComplete(IMqttDeliveryToken token) {
		// Do nothing.
	}

	@Override
	protected void doPublish(String topic, String ip) throws IOException {
		MqttClient mqttClient = this.client;
		if (mqttClient == null) {
			throw new IOException();
		}
		try {
			mqttClient.publish(topic, new MqttMessage(ip.getBytes()));
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

	@Override
	protected synchronized void doConnect() throws IOException {
		if (this.client != null) {
			disconnect();
		}
		MqttClient client;
		try {
			client = new MqttClient(this.uri, this.clientID);
			client.connect();
			this.client = client;
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

	@Override
	protected synchronized void doDisconnect() {
		MqttClient client = this.client;
		this.client = null;
		if (client != null) {
			try {
				client.close();
			} catch (MqttException e) {
				Messages.LOGGER.log(Level.FINE, Messages.CATEGORY, Messages.ERROR_UNKNOWN, e);
			}
		}
	}

	@Override
	public boolean isConnected() {
		return super.isConnected() && this.client != null;
	}
}
