/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.background;

import ej.annotation.Nullable;
import ej.microui.display.GraphicsContext;
import ej.microui.display.Image;
import ej.style.container.AlignmentHelper;
import ej.style.container.Rectangle;

/**
 * Draws an image on the background.
 */
public class SimpleImageBackground extends AbstractImageBackground {

	/**
	 * Creates a simple image background.
	 */
	public SimpleImageBackground() {
		setFillColor(true);
	}

	/**
	 * Creates a simple image background specifying its image and alignment.
	 *
	 * @param image
	 *            the background image to set.
	 * @param alignment
	 *            the alignment to set.
	 * @param fillColor
	 *            <code>true</code> to fill with color before drawing the image, <code>false</code> otherwise.
	 */
	public SimpleImageBackground(Image image, int alignment, boolean fillColor) {
		super(image, alignment, fillColor);
	}

	@Override
	protected void drawImage(GraphicsContext g, Rectangle bounds) {
		Image image = getImage();
		if (image != null) {
			int alignment = getAlignment();
			int x = AlignmentHelper.computeXLeftCorner(image.getWidth(), 0, bounds.getWidth(), alignment);
			int y = AlignmentHelper.computeYTopCorner(image.getHeight(), 0, bounds.getHeight(), alignment);
			g.drawImage(image, x, y, GraphicsContext.LEFT | GraphicsContext.TOP);
		}
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		return (obj instanceof SimpleImageBackground && super.equals(obj));
	}

	@Override
	public int hashCode() {
		// Use super implementation of hashCode() since there is no field.
		return super.hashCode();
	}

}
