/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.Selector;

/**
 * A not selector selects every element that is not selected by its wrapped selector.
 * <p>
 * Equivalent to <code>:not()</code> selector in CSS. Its specificity is the same as its wrapped selector.
 *
 * @see SpecificityHelper
 */
public class NotSelector implements Selector {

	private final Selector selector;

	/**
	 * Creates a not selector.
	 *
	 * @param selector
	 *            the wrapped selector.
	 */
	public NotSelector(Selector selector) {
		this.selector = selector;
	}

	@Override
	public boolean fit(Element element) {
		return !this.selector.fit(element);
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof NotSelector) {
			NotSelector other = (NotSelector) obj;
			return this.selector.equals(other.selector);
		}
		return false;
	}

	@Override
	public int getSpecificity() {
		return this.selector.getSpecificity();
	}

	@Override
	public int hashCode() {
		return this.selector.hashCode();
	}
}
