/*
 * Java
 *
 * Copyright 2018-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.monosandbox.application;

import ej.basictool.ArrayTools;
import ej.util.message.Level;
import ej.wadapps.application.Application;
import ej.wadapps.application.ApplicationLifecycleListener;
import ej.wadapps.application.ApplicationManager;
import ej.wadapps.monosandbox.util.WadappsMonoSandboxMessage;

/**
 * Application manager without sandboxing.
 */
public abstract class AbstractApplicationManagerMonoSandbox implements ApplicationManager {

	private ApplicationLifecycleListener[] listeners;

	/**
	 * Creates an application manager without sandboxing.
	 */
	public AbstractApplicationManagerMonoSandbox() {
		this.listeners = new ApplicationLifecycleListener[0];
	}

	@SuppressWarnings("null")
	@Override
	public void addApplicationLifecycleListener(ApplicationLifecycleListener applicationLifecycleListener) {
		this.listeners = ArrayTools.add(this.listeners, applicationLifecycleListener);
	}

	@SuppressWarnings("null")
	@Override
	public void removeApplicationLifecycleListener(ApplicationLifecycleListener applicationLifecycleListener) {
		this.listeners = ArrayTools.remove(this.listeners, applicationLifecycleListener);
	}

	void onStart(Application application) {
		for (ApplicationLifecycleListener listener : this.listeners) {
			try {
				listener.onStart(application);
			} catch (Throwable e) {
				WadappsMonoSandboxMessage.LOGGER.log(Level.INFO, WadappsMonoSandboxMessage.CATEGORY,
						WadappsMonoSandboxMessage.USER_ERROR, e);
			}
		}
	}

	void onStop(Application application) {
		for (ApplicationLifecycleListener listener : this.listeners) {
			try {
				listener.onStop(application);
			} catch (Throwable e) {
				WadappsMonoSandboxMessage.LOGGER.log(Level.INFO, WadappsMonoSandboxMessage.CATEGORY,
						WadappsMonoSandboxMessage.USER_ERROR, e);
			}
		}
	}

}
