/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.wadapps.registry;

import ej.components.dependencyinjection.SystemPropertiesServiceLoader;

/**
 * The factory that retrieves the unique instance of the {@link SharedRegistry} set by the system.
 */
public class SharedRegistryFactory {

	/**
	 * See explicit clinit dependency to <code>System.getProperty()</code>.
	 */
	private static final SharedRegistry SHARED_REGISTRY = new SystemPropertiesServiceLoader()
			.getService(SharedRegistry.class);

	private SharedRegistryFactory() {
	}

	/**
	 * Gets the global registry.
	 *
	 * @return the global registry.
	 */
	public static SharedRegistry getSharedRegistry() {
		return SHARED_REGISTRY;
	}

}
