/*
 * Java
 *
 * Copyright 2015-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.service;

import ej.service.loader.SystemPropertiesServiceLoader;

/**
 * The factory that retrieves the unique instance of the {@link SharedServiceRegistry} sets by the system.
 */
public class SharedServiceFactory {

	private static final SharedServiceRegistry SHARED_SERVICE_REGISTRY = getSharedServiceRegistryInstance();

	private SharedServiceFactory() {
	}

	private static SharedServiceRegistry getSharedServiceRegistryInstance() {
		SharedServiceRegistry sharedServiceRegistry = new SystemPropertiesServiceLoader()
				.getService(SharedServiceRegistry.class);
		if (sharedServiceRegistry == null) {
			throw new IllegalArgumentException();
		}
		return sharedServiceRegistry;
	}

	/**
	 * Gets the shared service registry.
	 *
	 * @return the shared service registry.
	 */
	public static SharedServiceRegistry getSharedServiceRegistry() {
		return SHARED_SERVICE_REGISTRY;
	}

}
